@@ -12,6 +12,11 @@
 ##' a temporary view with name ".rpostgis_TEMPview" is used only
 ##' within the function execution. In this mode, the other arguments can be used 
 ##' normally to modify the Spatial* object returned from the query.
+##' 
+##' Definitions for tables written in "data frame mode" are automatically
+##' applied using this function, including proj4strings of the \code{Spatial*}-class
+##' object. Note that if the proj4string of the original dataset is not found to 
+##' be equivalent to the database proj4string (using \code{pgSRID}), it will not be applied.
 ##'
 ##' @param conn A connection object to a PostgreSQL database
 ##' @param name A character string specifying a PostgreSQL schema and
@@ -19,8 +24,7 @@
 ##'     c("schema","table")})
 ##' @param geom The name of the geometry/(geography) column. (Default = \code{"geom"})
 ##' @param gid Name of the column in \code{name} holding the IDs. Should be
-##'     unique if additional columns of unique data are being
-##'     appended. \code{gid=NULL} (default) automatically creates a
+##'     unique for each record to return. \code{gid=NULL} (default) automatically creates a
 ##'     new unique ID for each row in the \code{sp} object.
 ##' @param other.cols Names of specific columns in the table to
 ##'     retrieve, in a character vector (e.g. \code{other.cols=c("col1","col2")}.)
@@ -31,13 +35,23 @@
 ##' @param clauses character, additional SQL to append to modify select
 ##'     query from table. Must begin with an SQL clause (e.g., "WHERE ...",
 ##'     "ORDER BY ...", "LIMIT ..."); see below for examples.
+##' @param boundary \code{sp} object or numeric. A Spatial* object,
+##'     whose bounding box will be used to select geometries
+##'     to import. Alternatively, four numbers
+##'     (e.g. \code{c([top], [bottom], [right], [left])}) indicating the
+##'     projection-specific limits with which to subset spatial data. \code{boundary = NULL}
+##'     (default) will not subset by spatial extent.
+##'     Note this is not a true 'clip'- all features intersecting the
+##'     bounding box with be returned unmodified.
 ##' @param query character, a full SQL query including a geometry column. 
 ##'     For use with query mode only (see details).
 ##' @return sp-class (SpatialPoints*, SpatialMultiPoints*, SpatialLines*, or SpatialPolygons*)
+##' @importFrom sp proj4string
 ##' @export
-##' @author David Bucklin \email{dbucklin@@ufl.edu}
+##' @author David Bucklin \email{david.bucklin@@gmail.com}
 ##' @author Mathieu Basille \email{basille@@ufl.edu}
 ##' 
+##' 
 ##' @examples
 ##' \dontrun{
 ##' ## Retrieve a Spatial*DataFrame with all data from table
@@ -52,7 +66,7 @@
 ##' ## Return a Spatial*-only (no data frame), 
 ##' ## retaining id from table as rownames and with a subset of the data
 ##' pgGetGeom(conn, c("schema", "roads"), geom = "roadgeom", gid = "road_ID",
-##'     other.cols = FALSE, clauses  = "WHERE field = 'highway'")
+##'     other.cols = FALSE, clauses  = "WHERE road_type = 'highway'")
 ##' ## Query mode
 ##' pgGetGeom(conn, query = "SELECT r.gid as id, ST_Buffer(r.geom, 100) as geom 
 ##'                            FROM
@@ -63,27 +77,43 @@
 ##' }
 
 pgGetGeom <- function(conn, name, geom = "geom", gid = NULL, 
-    other.cols = TRUE, clauses = NULL, query = NULL) {
+    other.cols = TRUE, clauses = NULL, boundary = NULL, query = NULL) {
     dbConnCheck(conn)
     if (!suppressMessages(pgPostGIS(conn))) {
         stop("PostGIS is not enabled on this database.")
     }
     if (!is.null(query)) {
       if (missing(name)) name <- NULL
-      ret <- pgGetGeomQ(conn, query, name = name, geom = geom, gid = gid, 
-                        other.cols = other.cols, clauses = clauses)
+      ret <- pgGetGeomQ(conn, query = query, name = name, geom = geom, gid = gid, 
+                        other.cols = other.cols, clauses = clauses, boundary = boundary)
       if (is.null(ret)) stop("Query retrieval failed.", call. = FALSE) else return(ret)
     }
     ## Check and prepare the schema.name
     nameque <- paste(dbTableNameFix(conn,name), collapse = ".")
-    namechar <- gsub("\"\"", "\"", gsub("'", "''", paste(gsub("^\"|\"$", 
-        "", dbTableNameFix(conn,name)), collapse = ".")))
     
-    geomque <- pgCheckGeom(conn, namechar, geom)
+    geomque <- pgCheckGeom(conn, name, geom)
+    
+    if (!is.null(boundary)) {
+      ## get srid for boundary box - could be incorrect only if clauses are used to specify a subset by SRID
+      srid <- pgGetSRID(conn, name, geom)
+      if (typeof(boundary) != "double") {
+            boundary <- c(boundary@bbox[2, 2], boundary@bbox[2,
+                1], boundary@bbox[1, 2], boundary@bbox[1, 1])
+      }
+      b.clause <- paste0(" AND ST_Intersects(",
+              geomque, ",ST_SetSRID(ST_GeomFromText('POLYGON((", boundary[4],
+              " ", boundary[1], ",", boundary[4], " ", boundary[2],
+              ",\n  ", boundary[3], " ", boundary[2], ",", boundary[3],
+              " ", boundary[1], ",", boundary[4], " ", boundary[1],
+              "))'),", srid[1], "))")
+    } else {
+      b.clause <- NULL
+    }
     
     ## prepare clauses
-    if (!is.null(clauses)) {
-      clauses <- sub("^where", "AND", clauses, ignore.case = TRUE) 
+    if (!is.null(clauses) | !is.null(b.clause)) {
+      clauses <- paste(b.clause, sub("^where", "AND", sub(";$","", sub("\\s+$","",clauses)),
+                                     ignore.case = TRUE),collapse = "\n") 
       } else {
         if (".db_pkid" %in% dbTableInfo(conn,name)$column_name) {
           clauses <- "ORDER BY \".db_pkid\""
@@ -94,7 +124,8 @@ pgGetGeom <- function(conn, name, geom = "geom", gid = NULL,
         other.cols <- paste(DBI::dbQuoteIdentifier(conn, other.cols), 
             collapse = ",")
     } else {
-        if (other.cols) {
+        if (other.cols & sum(!dbTableInfo(conn,
+                            name)$column_name %in% c(".R_rownames",".db_pkid", gid, geom)) > 0) {
             other.cols <- "*"
         } else {
             other.cols <- NULL
@@ -106,6 +137,7 @@ pgGetGeom <- function(conn, name, geom = "geom", gid = NULL,
                         ") as geo FROM ", nameque, " WHERE ", geomque, " IS NOT NULL ", 
                         clauses, ") a;")
     typ <- dbGetQuery(conn, tmp.query)$type
+
     # assign to correct function
     if (length(typ) == 0) {
         stop("No geometries found.")
@@ -113,21 +145,19 @@ pgGetGeom <- function(conn, name, geom = "geom", gid = NULL,
         if (typ %in% c("ST_Point", "ST_MultiPoint")) {
             ret <- pgGetPts(conn, name, geom, gid, other.cols, 
                 clauses)
+            if (typ == "ST_MultiPoint") mp <- "Multi" else mp<-NULL
             message(paste0("Returning ", sub("...", "", typ), 
-                " types in SpatialPoints*-class."))
-            return(ret)
+                " types in Spatial",mp,"Points*-class."))
         } else if (typ %in% c("ST_LineString", "ST_MultiLineString")) {
             ret <- pgGetLines(conn, name, geom, gid, other.cols, 
                 clauses)
             message(paste0("Returning ", sub("...", "", typ), 
                 " types in SpatialLines*-class."))
-            return(ret)
         } else if (typ %in% c("ST_Polygon", "ST_MultiPolygon")) {
             ret <- pgGetPolys(conn, name, geom, gid, other.cols, 
                 clauses)
             message(paste0("Returning ", sub("...", "", typ), 
                 " types in SpatialPolygons*-class."))
-            return(ret)
         } else {
             stop(paste0("Geometry type ", typ, " not supported."))
         }
@@ -136,13 +166,34 @@ pgGetGeom <- function(conn, name, geom = "geom", gid = NULL,
             collapse = ", "), "). Use
                   \"clauses\" to modify query to select only one geometry type."))
     }
+    
+    ## get df mode proj4string
+    defs <- dbGetDefs(conn, name)
+    if (length(defs) > 0 & geom %in% defs$nms) {
+      p4s <- defs$atts[defs$nms == geom]
+      if (p4s != "NA") {
+        p4s <- CRS(p4s, doCheckCRSArgs = FALSE)
+        # change (exact) proj4string if equivalent to existing
+        suppressMessages({
+          suppressWarnings({
+          if (any(pgSRID(conn, ret@proj4string) %in% pgSRID(conn, p4s))) {
+            sp::proj4string(ret) <- p4s
+          }
+          })
+        })
+      }
+      return(ret)
+    } else {
+      return(ret)
+    }
+    
 }
 
 ## pgGetPts
 
 ##' Load a PostGIS point geometry from a PostgreSQL table/view into R.
 ##' @return Spatial(Multi)PointsDataFrame or Spatial(Multi)Points
-##' @author David Bucklin \email{dbucklin@@ufl.edu}
+##' @author David Bucklin \email{david.bucklin@@gmail.com}
 ##' @author Mathieu Basille \email{basille@@ufl.edu}
 ##' @importFrom sp CRS
 ##' @importFrom sp SpatialPoints
@@ -173,9 +224,7 @@ pgGetPts <- function(conn, name, geom = "geom", gid = NULL, other.cols = "*",
     ## prepare additional clauses
     clauses<-sub("^where", "AND",clauses, ignore.case = TRUE)
     ## prepare geom column
-    namechar <- gsub("\"\"", "\"", gsub("'", "''", paste(gsub("^\"|\"$", 
-        "", dbTableNameFix(conn,name)), collapse = ".")))
-    geomque <- pgCheckGeom(conn, namechar, geom)
+    geomque <- pgCheckGeom(conn, name, geom)
     ## If ID not specified, set it to generate row numbers
     if (is.null(gid)) {
         if (".R_rownames" %in% dbTableInfo(conn,name)$column_name) {
@@ -236,9 +285,11 @@ pgGetPts <- function(conn, name, geom = "geom", gid = NULL, other.cols = "*",
         if (!is.null(other.cols)) {
             cols <- colnames(dbData)[4:length(colnames(dbData))]
             cols <- cols[!(cols %in% c(geom))]
+            
+            dfr <- dbData[4:length(colnames(dbData))][cols]
             # column definitions
-            suppressMessages(
-              dfr<-dbReadDataFrame(conn, name, df = dbData[,4:length(colnames(dbData))][cols])
+            if (gid == "\".R_rownames\"") suppressMessages(
+              dfr<-dbReadDataFrame(conn, name, df = dfr)
             )
             sp <- sp::SpatialPointsDataFrame(sp, dfr,
                 match.ID = TRUE)
@@ -264,9 +315,10 @@ pgGetPts <- function(conn, name, geom = "geom", gid = NULL, other.cols = "*",
         if (!is.null(other.cols)) {
             cols <- colnames(dbData)[3:length(colnames(dbData))]
             cols <- cols[!(cols %in% c(geom))]
+            dfr <- dbData[3:length(colnames(dbData))][cols]
             # column definitions
-            suppressMessages(
-              dfr<-dbReadDataFrame(conn, name, df = dbData[,3:length(colnames(dbData))][cols])
+            if (gid == "\".R_rownames\"") suppressMessages(
+              dfr<-dbReadDataFrame(conn, name, df = dfr)
             )
             sp <- sp::SpatialMultiPointsDataFrame(tt, dfr,
                 proj4string = proj4)
@@ -279,7 +331,7 @@ pgGetPts <- function(conn, name, geom = "geom", gid = NULL, other.cols = "*",
 ## pgGetLines
 
 ##' Load a PostGIS linestring geometry from a PostgreSQL table/view into R.
-##' @author David Bucklin \email{dbucklin@@ufl.edu}
+##' @author David Bucklin \email{david.bucklin@@gmail.com}
 ##' @author Mathieu Basille \email{basille@@ufl.edu}
 ##' @importFrom sp CRS
 ##' @importFrom sp SpatialLines
@@ -308,9 +360,7 @@ pgGetLines <- function(conn, name, geom = "geom", gid = NULL,
     clauses<-sub("^where", "AND",clauses, ignore.case = TRUE)
     
     ## prepare geom column
-    namechar <- gsub("\"\"", "\"", gsub("'", "''", paste(gsub("^\"|\"$", 
-        "", dbTableNameFix(conn,name)), collapse = ".")))
-    geomque <- pgCheckGeom(conn, namechar, geom)
+    geomque <- pgCheckGeom(conn, name, geom)
     ## Check gid
     if (is.null(gid)) {
         if (".R_rownames" %in% dbTableInfo(conn,name)$column_name) {
@@ -373,8 +423,9 @@ pgGetLines <- function(conn, name, geom = "geom", gid = NULL,
         cols <- colnames(dfTemp)[3:length(colnames(dfTemp))]
         cols <- cols[!(cols %in% c(geom))]
         # column definitions
-        suppressMessages(
-            dfr<-dbReadDataFrame(conn, name, df = dfTemp[,3:length(colnames(dfTemp))][cols])
+        dfr <- dfTemp[3:length(colnames(dfTemp))][cols]
+        if (gid == "\".R_rownames\"") suppressMessages(
+            dfr<-dbReadDataFrame(conn, name, df = dfr)
         )
         
         spdf <- sp::SpatialLinesDataFrame(Sline, dfr)
@@ -385,7 +436,7 @@ pgGetLines <- function(conn, name, geom = "geom", gid = NULL,
 ## pgGetPolys
 
 ##' Load a PostGIS polygon geometry from a PostgreSQL table/view into R.
-##' @author David Bucklin \email{dbucklin@@ufl.edu}
+##' @author David Bucklin \email{david.bucklin@@gmail.com}
 ##' @author Mathieu Basille \email{basille@@ufl.edu}
 ##' @importFrom sp CRS
 ##' @importFrom sp SpatialPolygons
@@ -413,9 +464,7 @@ pgGetPolys <- function(conn, name, geom = "geom", gid = NULL,
     ## prepare additional clauses
     clauses<-sub("^where", "AND",clauses, ignore.case = TRUE)
     ## prepare geom column
-    namechar <- gsub("\"\"", "\"", gsub("'", "''", paste(gsub("^\"|\"$", 
-        "", dbTableNameFix(conn,name)), collapse = ".")))
-    geomque <- pgCheckGeom(conn, namechar, geom)
+    geomque <- pgCheckGeom(conn, name, geom)
     ## Check gid
     if (is.null(gid)) {
         if (".R_rownames" %in% dbTableInfo(conn,name)$column_name) {
@@ -476,9 +525,10 @@ pgGetPolys <- function(conn, name, geom = "geom", gid = NULL,
     } else {
         cols <- colnames(dfTemp)[3:length(colnames(dfTemp))]
         cols <- cols[!(cols %in% c(geom))]
+        dfr <- dfTemp[3:length(colnames(dfTemp))][cols]
         # column definitions
-        suppressMessages(
-            dfr<-dbReadDataFrame(conn, name, df = dfTemp[,3:length(colnames(dfTemp))][cols])
+        if (gid == "\".R_rownames\"") suppressMessages(
+            dfr<-dbReadDataFrame(conn, name, df = dfr)
         )
         spdf <- sp::SpatialPolygonsDataFrame(Spol, dfr)
         return(spdf)
@@ -501,9 +551,10 @@ pgGetPolys <- function(conn, name, geom = "geom", gid = NULL,
 pgGetGeomQ <- function(conn, query, name = NULL, ...) {
     # set view name
     if (is.null(name)) {
-        name <- ".rpostgis_TEMPview"
+        name <- dbTableNameFix(conn, ".rpostgis_TEMPview", as.identifier = FALSE)
         keep <- FALSE
     } else {
+        name <- dbTableNameFix(conn, name , as.identifier = FALSE)
         keep <- TRUE
     }
     dbExecute(conn, "BEGIN;")
@@ -512,7 +563,7 @@ pgGetGeomQ <- function(conn, query, name = NULL, ...) {
     try({
         prequery <- paste0("CREATE OR REPLACE VIEW ", paste(dbQuoteIdentifier(conn, 
             name), collapse = "."), " AS ")
-        if (sub(".*(?=.$)", "", sub("\\s+$", "", query), perl = T) == 
+        if (sub(".*(?=.$)", "", sub("\\s+$", "", query), perl = TRUE) == 
             ";") {
             post <- NULL
         } else {
@@ -521,13 +572,17 @@ pgGetGeomQ <- function(conn, query, name = NULL, ...) {
         q <- paste0(prequery, query, post)
         dbExecute(conn, q)
         geo <- pgGetGeom(conn, name = name, ...)
+        
     })
     # rollback on failed/not saving view, else commit
     if (is.null(geo)) {
         dbExecute(conn, "ROLLBACK;")
     } else {
-        if (!keep) 
-            dbExecute(conn, "ROLLBACK;") else dbExecute(conn, "COMMIT;")
+        if (!keep) { dbExecute(conn, "ROLLBACK;") } else {
+          dbExecute(conn, "COMMIT;")
+          message(paste0("Created view ",paste(dbQuoteIdentifier(conn, 
+            name), collapse = "."),"."))
+        }
     }
     return(geo)
 }
