@@ -1,26 +1,24 @@
 ## pgListGeom
 
-##' List geometries.
+##' List geometries/rasters
 ##'
-##' List all geometry/(geography) objects available in a PostGIS database.
+##' List all geometry/(geography) or raster columns available in a PostGIS database.
 ##'
 ##' @param conn A PostgreSQL database connection.
-##' @param display Logical. Whether to display the query (defaults to
-##'     \code{TRUE}).
-##' @param exec Logical. Whether to execute the query (defaults to
-##'     \code{TRUE}).
-##' @param geog Logical. Whether to include PostGIS geography-type 
+##' @param geog Logical. For pgListGeom, whether to include PostGIS geography-type 
 ##'     columns stored in the database
 ##' @return If \code{exec = TRUE}, a data frame with schema, table,
-##'     geometry/(geography) column, and geometry/(geography) type.
-##' @author David Bucklin \email{dbucklin@@ufl.edu}
+##'     geometry/(geography) or raster (for pgListRast) column, and geometry/(geography) type.
+##' @author David Bucklin \email{david.bucklin@@gmail.com}
 ##' @export
 ##' @examples
 ##' \dontrun{
 ##' pgListGeom(conn)
+##' 
+##' pgListRast(conn)
 ##' }
 
-pgListGeom <- function(conn, display = TRUE, exec = TRUE, geog = FALSE) {
+pgListGeom <- function(conn, geog = TRUE) {
     dbConnCheck(conn)
     ## Check if PostGIS is enabled
     if (!suppressMessages(pgPostGIS(conn))) {
@@ -45,15 +43,38 @@ pgListGeom <- function(conn, display = TRUE, exec = TRUE, geog = FALSE) {
         "    f_table_name AS table_name,", "    f_geometry_column AS geom_column,",
         "    type AS geometry_type,", "    'GEOMETRY'::character(8) AS type", "FROM geometry_columns ", end , 
         sep = "\n")
-    ## Display the query
-    if (display) {
-        message(paste0("Query ", ifelse(exec, "", "not "), "executed:"))
-        message(tmp.query)
-        message("--")
-    }
-    ## Execute the query and return it if successful
-    if (exec) {
-        tab <- dbGetQuery(conn, tmp.query)
-        return(tab)
-    }
+    
+    tab <- dbGetQuery(conn, tmp.query)
+    return(tab)
 }
+
+
+# pgListRast
+
+##' @rdname pgListGeom
+##' @export
+
+pgListRast <- function(conn) {
+    dbConnCheck(conn)
+    ## Check if PostGIS is enabled
+    if (!suppressMessages(pgPostGIS(conn))) {
+        stop("PostGIS is not enabled on this database.")
+    }
+    ## SQL Query:
+    ## --
+    ## SELECT
+    ##     r_table_schema AS schema_name,
+    ##     r_table_name AS table_name,
+    ##     r_raster_column AS raster_column
+    ## FROM raster_columns;
+    ## --
+
+    tmp.query <- paste("SELECT", "    r_table_schema AS schema_name,",
+        "    r_table_name AS table_name,", "    r_raster_column AS raster_column",
+        "FROM raster_columns;",
+        sep = "\n")
+    
+    ## Execute the query and return it if successful
+    tab <- dbGetQuery(conn, tmp.query)
+    return(tab)
+}
\ No newline at end of file
